/* Led Cycling
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Image Loader.

The Imote2 Image Loader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The Imote2 Image Loader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the Imote2 Image Loader. If not, see <http://www.gnu.org/licenses/>.
*/
#include <os_arm_bare_registers_def.h>
#include <pmic.h>
#include <inttypes.h>

static inline void wait() {
	OSCR0 = 0;
	/* wait half a second */
	while(OSCR0 < 1625000);
}

static uint8_t readPMIC(const uint8_t address) {
	uint8_t ret;
	
	PICR = ICR_IUE | ICR_SCLE;
	PIDBR = PMIC_SLAVE_ADDR<<1;
	PICR |= ICR_START | ICR_TB;
	while(PICR & ICR_TB);
	
	//actually send the address terminated with a STOP
	PIDBR = address;
	PICR &= ~ICR_START;
	PICR |= ICR_STOP | ICR_TB;
	while(PICR & ICR_TB);
	PICR &= ~ICR_STOP;
	
	//actually request the read of the data
	PIDBR = PMIC_SLAVE_ADDR << 1 | 1;
	PICR |= ICR_START | ICR_TB;
	while(PICR & ICR_TB);
	PICR &= ~ICR_START;
	
	PICR |= ICR_STOP | ICR_ACKNAK| ICR_TB;
	while(PICR & ICR_TB);
	ret = PIDBR;
	
	PICR &= ~(ICR_STOP|ICR_ACKNAK|ICR_IUE | ICR_SCLE);
	
	PCFR = PCFR_PI2C_EN | PCFR_FVC | PCFR_DC_EN;
	return ret;
}

static inline bool buttonPressed() {
	static bool firstPress = TRUE;
	if (GEDR(1) & GPIO_BIT(1)) {
		GEDR(1) = GPIO_BIT(1);
		uint8_t event = readPMIC(PMIC_EVENTA);
		readPMIC(PMIC_EVENTB);
		readPMIC(PMIC_EVENTC);
		readPMIC(PMIC_EVENTA);
		readPMIC(PMIC_EVENTB);
		readPMIC(PMIC_EVENTC);
		firstPress = !firstPress;
		return event == 1 && firstPress;
	}
	return FALSE;
}

void _excLedCycle(bool keyPress) {
	static int curLed = 3;
	static int ledMask = 0;
	
	buttonPressed();
	while (keyPress == FALSE || !buttonPressed()) {
		wait();
		--curLed;
		ledMask |= (1 << 7) << curLed;
		if (!curLed) curLed = 3;
		GPCR3 = ledMask;
		
		wait();
		--curLed;
		ledMask &= ~((1 << 7) << curLed);
		if (!curLed) curLed = 3;
		GPSR3 = ((1 << 7) | (1 << 8) | (1 << 9)) & ~ledMask;
	}
}

